/*
 * osdthumbview.h
 *
 * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#ifndef OSDTHUMBVIEW_H
#define OSDTHUMBVIEW_H

#include <QList>
#include <QTime>

#include "osdthumbitem.h"



class TimeLine : public QObject
{
	Q_OBJECT
public:
	TimeLine() { steps=0; range=0; scroll=0; }
	void setDuration( int ms ) { duration=ms; }
	int currentPos() { return scroll; }
	void setRange( int r ) {
		if ( r<0 )
			range=0;
		else
			range=r;
		scroll = steps = 0;
	}
	int getDirection() {
		if ( steps<0 )
			return -1;
		else
			return 1;
	}
	void stop() { running=false; steps=0; }
	void stepForward() {
		qreal speed;
		qreal val = value( speed );
		if ( val>0.5 || steps==-1 )
			return;
		if ( (scroll+steps+1)>range )
			return;
		++steps;
		if ( !running )
			start();
	}
	void stepBackward() {
		qreal speed;
		qreal val = value( speed );
		if ( val>0.5 || steps==1 )
			return;
		if ( (scroll+steps-1)<0 )
			return;
		--steps;
		if ( !running )
			start();
	}
	bool isRunning() { return running; }
	qreal currentValue() {
		qreal speed;
		qreal val = value( speed );
		while ( val>1.0 ) {
			if ( steps>0 ) {
				--steps;
				++scroll;
			}
			else if ( steps<0 ) {
				++steps;
				--scroll;
			}
			if ( steps==0 )
				return -1.0;
			else {
				val-=1.0;
				timePassed+=(int)speed;
			}
		}
		if ( steps<0 )
			val = 1.0-val;
		return val;
	}

private:
	void start() { timePassed=0; running=true; timer.start(); emit startRepaint(); }
	qreal value( qreal &v ) {
		v = (qreal)duration/(qreal)qMin( 4, qAbs(steps) );
		return (qreal)(timer.elapsed()-timePassed)/v;
	}

	int duration;
	QTime timer;
	int steps;
	bool running;
	int range, scroll;
	int timePassed;

signals:
	void startRepaint();
};



class ThumbRect
{
public:
	QRectF rect;
	qreal opacity;
};



class OsdThumbView : public OsdObject
{
	Q_OBJECT
public:
	OsdThumbView( OsdConfig *conf );
	~OsdThumbView();
	void resize( int wi, int h );
	void clear();
	void append( OsdThumbItem *it );
	void updateThumb( const QString &s );

	bool mouseClicked( QMouseEvent *e );
	void wheelEvent( QWheelEvent* );
	void paint();

private:
	int setCoords( bool ended=false );

	QList<OsdThumbItem*> thumbs;
	QRectF clipRect, textRect;
	ThumbRect currentThumb;
	ThumbRect leftThumb[4];
	ThumbRect rightThumb[4];
	float scale, incx, incy;
	float currentRightInc, currentLeftInc, currentScale;
	TimeLine timeLine;

private slots:
	void finished();

signals:
	void startRepaint();
	void stopRepaint();
};

#endif // OSDTHUMBVIEW_H
