/*
 * osdvscrollbar.cpp
 *
 * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#include "osdvscrollbar.h"



OsdVScrollBar::OsdVScrollBar( OsdConfig *conf ) : OsdObject( conf )
{
	pos = 0;
	range = 1;
	scroll = false;
	buttonLen = MINSCROLLBARBUTTONSIZE;
	buttonOffset = 0;
	type = Vertical;
}



OsdVScrollBar::~OsdVScrollBar()
{
}



void OsdVScrollBar::mouseMove( QPoint &p )
{
	if ( !scroll )
		return;
	int off, tmp;
	if ( type==Vertical ) {
		off = p.y()-scrollStartPos;
		tmp = off*(range-rect.height())/(rect.height()-buttonLen) + scrollOldPos;
	}
	else {
		off = p.x()-scrollStartPos;
		tmp = off*(range-rect.width())/(rect.width()-buttonLen) + scrollOldPos;
	}

	if ( tmp<0 )
		tmp = 0;
	emit scrolled( tmp );
}



bool OsdVScrollBar::mouseClicked( QMouseEvent *e )
{
	if ( !inMe( e->x(), e->y() ) )
		return false;
	if ( type==Vertical ) {
		if ( e->y()<(rect.y()+buttonOffset) || e->y()>(rect.y()+buttonOffset+buttonLen) )
			return false;
		scrollOldPos = pos;
		scrollStartPos = e->y();
	}
	else {
		if ( e->x()<(rect.x()+buttonOffset) || e->x()>(rect.x()+buttonOffset+buttonLen) )
			return false;
		scrollOldPos = pos;
		scrollStartPos = e->x();
	}

	return scroll=true;
}



void OsdVScrollBar::mouseReleased( QMouseEvent *e )
{
	Q_UNUSED(e);
	scroll = false;
}



void OsdVScrollBar::paint()
{
	if ( type==Vertical ) {
		if ( rect.height()<=0 )
			return;
		//glScissor( rect.x(), rect.y(), rect.width(), rect.height() );
		//glEnable( GL_SCISSOR_TEST );
		if ( range>rect.height() ) {
			buttonLen = rect.height()*rect.height()/range;
			if ( buttonLen<MINSCROLLBARBUTTONSIZE )
				buttonLen = MINSCROLLBARBUTTONSIZE;
			buttonOffset = (pos*(rect.height()-buttonLen))/(range-rect.height());
		}
		else {
			buttonOffset = 0;
			buttonLen = 0;
		}

		config->func->drawQuad( rect, config->scrollBarBackgroundColor() );
		QRect r( rect.x(), rect.y()+buttonOffset, rect.width(), buttonLen);
		config->func->drawVerticalGradient( r, config->scrollBarGrad0Color(), config->scrollBarGrad1Color() );

		//glDisable( GL_SCISSOR_TEST );
	}
	else {
		if ( rect.width()<=0 )
			return;
		//glScissor( rect.x(), rect.y(), rect.width(), rect.height() );
		//glEnable( GL_SCISSOR_TEST );

		if ( range>rect.width() ) {
			buttonLen = MINSCROLLBARBUTTONSIZE;
			buttonOffset = (pos*(rect.width()-buttonLen))/(range-rect.width());
		}
		else {
			buttonOffset = 0;
			//buttonLen = 0;
		}

		config->func->drawQuad( rect, config->scrollBarBackgroundColor() );
		QRect r( rect.x()+buttonOffset, rect.y(), buttonLen, rect.height() );
		config->func->drawHorizontalGradient( r, config->scrollBarGrad0Color(), config->scrollBarGrad1Color() );

		//glDisable( GL_SCISSOR_TEST );
	}
}
